from datetime import datetime


def valiDate(date: str, string: str = "%Y-%m-%d") -> bool:
    try:
        if date != datetime.strptime(date, string).strftime(string):
            raise ValueError

        return True

    except ValueError:
        return False


def strToNum(string: str) -> "int, float, str":
    """
    Optionally converts a string to a number.
    """

    try:
        number = float(string)

        if number == int(number):
            return int(number)

        return number

    except ValueError:
        return string


def parser(prompt: list[str]) -> tuple[list[str], dict[str, str], list[str]]:
    """
    Prompt parser.
    """

    instructions: list[str] = []
    sdOpts: dict[str, str] = {}
    ddOpts: list[str] = []

    assert isinstance(prompt, list)

    # Single dash option skip flag.
    sdSkip: bool = False

    for j in range(len(prompt)):
        if sdSkip:
            sdSkip = False
            continue

        entry = prompt[j]
        assert isinstance(entry, str)

        # Double dash options.
        if len(entry) > 2:
            if entry[0] == entry[1] == "-":
                ddOpts.append(entry.replace("--", ""))

                continue

        # Single dash options.
        if len(entry) > 1:
            if entry[0] == "-":
                # Ignores negative numbers.
                try:
                    _ = float(entry)
                    continue

                except ValueError:
                    pass

                # Risks IndexError.
                try:
                    sdOpts[entry.replace("-", "", 1)] = strToNum(prompt[j + 1])
                    sdSkip = True

                except IndexError:
                    pass

                continue

        # Plain instructions.
        instructions.append(entry)

    if "debug" in ddOpts:
        print("instructions: " + ", ".join(instructions))
        print("sdOpts: " + ", ".join(sdOpts))
        print("ddOpts: " + ", ".join(ddOpts))

    return instructions, sdOpts, ddOpts



from __future__ import annotations

from datetime import datetime
from colorama import Fore, Style

from .utilities import parser, valiDate


class reminder:
    def __init__(self: reminder, prompt: list[str], old: reminder = None):
        _, sdOpts, _ = parser(prompt)

        try:
            # Checks title.
            assert "t" in sdOpts

            # Title.
            self.title: str = sdOpts["t"]

            # Checks priority.
            if "p" in sdOpts:
                assert isinstance(sdOpts["p"], int)
                assert 1 <= sdOpts["p"] <= 3

                # Priority.
                self.priority: int = sdOpts["p"]

            else:
                self.priority = 0

            # Checks description.
            if "de" in sdOpts:
                assert isinstance(sdOpts["de"], str)

                # Description.
                self.description: str = sdOpts["de"]

            else:
                self.description = ""

            # Checks date.
            if "da" in sdOpts:
                assert isinstance(sdOpts["da"], str)
                assert valiDate(sdOpts["da"])

                # Date.
                self.date: str = sdOpts["da"]

            else:
                self.date = ""

            # Checks time.
            if "ti" in sdOpts:
                assert isinstance(sdOpts["ti"], str)
                assert valiDate(sdOpts["ti"], "%H:%M")

                if not self.date:
                    self.date = datetime.now().strftime("%Y-%m-%d")

                # Time.
                self.time: str = sdOpts["ti"]

            else:
                self.time = ""

            # Flags.
            self.expired = False
            self.dismissed: bool = False
            self.confirmation: bool = True

            # Post-creation check.
            self.check()

            message = "Reminder created succesfully!"

        except AssertionError:
            message = "Reminder creation failed!"
            self.confirmation: bool = False

        print(message)
        print("-" * len(message))

    def __str__(self: reminder, index: int = -1) -> str:
        # Mark.
        mark = "\u25cf " if self.dismissed else "\u25ef "

        if index != -1:
            mark += "[{}] ".format(index)

        # Lenght of 'mark' in spaces.
        spaces = len(mark) * " "

        # Title.
        # Striked on dismissed reminders.
        if self.dismissed:
            title = Style.DIM + self.title + Style.RESET_ALL

        else:
            title = Style.BRIGHT + self.title + Style.RESET_ALL

        string = mark + title

        # Priority.
        if self.priority:
            priority = " " + Fore.RED + "!" * self.priority + Fore.RESET

            string += priority

        # Description.
        if self.description:
            description = Style.DIM + self.description + Style.RESET_ALL

            string += "\n" + spaces + description

        # Date.
        if self.date or self.time:
            date: str = Fore.GREEN if not self.expired else Fore.RED
            date += self.date + "{}{}" + Fore.RESET
            date = date.format(" " if self.date else "", self.time)

            string += "\n" + spaces + date

        # Finally returns string.
        return string

    def check(self: reminder) -> None:
        if self.date:
            date = datetime.strptime(self.date, "%Y-%m-%d")

            if self.time:
                time = datetime.strptime(self.time, "%H:%M")

            now = datetime.now()

            # Checks expiration.
            if date < now:
                if self.time:
                    if time.hour <= now.hour and time.minute < now.minute:
                        self.expired = True

                    else:
                        self.expired = False

                else:
                    self.expired = True

            else:
                self.expired = False

    def toggle(self: reminder) -> None:
        """
        Toggles itself.
        """

        self.dismissed = not self.dismissed



import pickle
import sys

from colorama import Fore, Style, init

from .help import help
from .files import getReminders, saveReminders
from .reminders import reminder
from .utilities import parser

# Colorama's initialization.
init()


# Defines promemoria's main function.
# This makes possible calling promemoria as a script.
def main() -> None:
    # Obtains instructions and options.
    instructions, sdOpts, ddOpts = parser(sys.argv)
    index: int = -1

    # Gets reminders.
    reminders: list[reminder] = getReminders()

    # Get reminder index, if present.
    if "i" in sdOpts:
        try:
            assert isinstance(sdOpts["i"], int)
            assert 0 < sdOpts["i"] <= len(reminders)

            index = sdOpts["i"] - 1

        except AssertionError:
            pass

    # Application name.
    print(Style.BRIGHT + "[promemoria]" + Style.RESET_ALL + "\n")

    # Helper.
    if "help" in instructions:
        print(help())

    # Creates a new reminder.
    elif "new" in instructions:
        newReminder = reminder(sys.argv)

        if newReminder.confirmation:
            reminders.append(newReminder)

            print("\n" + str(newReminder))

    # Delete all reminders.
    elif "clear" in instructions:
        reminders = []

        print("Your reminders have been deleted.")

    # Delete a reminder.
    elif "delete" in instructions:
        if index < 0:
            print(Fore.RED + "Syntax error." + Style.RESET_ALL)
            return -1

        message = "You have deleted a reminder."

        print(message)
        print("-" * len(message))

        print("\n" + str(reminders.pop(index)))

    # Toggle a reminder.
    elif "toggle" in instructions:
        if index < 0:
            print(Fore.RED + "Syntax error." + Style.RESET_ALL)
            return -1

        message = "You toggled a reminder."

        print(message)
        print("-" * len(message))

        reminders[index].toggle()
        print("\n" + str(reminders[index]))

    # No reminders.
    elif not len(reminders):
        hint = Style.BRIGHT + "promemoria new -t 'TITLE' ..." + Style.RESET_ALL

        print("You have no reminders.")
        print("Try creating one using " + hint)

    # Shows reminders by default.
    else:
        # Get printable reminders.
        if "all" not in ddOpts:
            printable = [rem for rem in reminders if not rem.dismissed]

        else:
            printable = reminders.copy()

        # Print reminders, if any.
        if len(printable):
            message: str = "You have {} reminder(s).".format(len(printable))

            print(message)
            print("-" * len(message))

            # Prints the list of reminders.
            for rem in printable:
                pIndex = reminders.index(rem)
                print("\n" + rem.__str__(pIndex + 1))

        else:
            print("Nothing to show.")

        if "all" in ddOpts:
            # Prints the number of completed reminders.
            completed: int = [rem.dismissed for rem in reminders].count(True)
            message: str = "{} completed.".format(completed)

            print("\n" + "-" * len(message))
            print(message)

    # Saves reminders.
    saveReminders(reminders)
